#!python
# Copyright (c) 2007 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

"""Invoke a suite of PHP tests, and parse the results.
"""

import sys, os

CREATE_NO_WINDOW = 0x8000000 # same as win32process.CREATE_NO_WINDOW

class KoHarnessPHPTests(object):
    def __init__(self, cmd, cwd=None):
        self._cmd = cmd
        if cwd is None:
            self._cwd = os.getcwd()
        else:
            self._cwd = cwd
        
    def run(self):
        import subprocess
        if sys.platform.startswith("win"):
            # Don't display the Windows GPF dialog if the invoked program dies.
            # See comp.os.ms-windows.programmer.win32
            #  How to suppress crash notification dialog?, Jan 14,2004 - Raymond Chen's response
            # (URL http://groups.google.com/group/comp.os.ms-windows.programmer.win32/tree/browse_frm/thread/eeade5ecfa04d000/658b412a62ef5d7a?hl=en&rnum=1&q=suppressing+%22has+encountered+a+problem+and+needs+to+close%22+windows&_done=%2Fgroup%2Fcomp.os.ms-windows.programmer.win32%2Fbrowse_frm%2Fthread%2Feeade5ecfa04d000%2F74397b6bf9405304%3Fhl%3Den%26lnk%3Dst%26q%3Dsuppressing%2B%2522has%2Bencountered%2Ba%2Bproblem%2Band%2Bneeds%2Bto%2Bclose%2522%2Bwindows%26#doc_41fe71be34d3614b)
            import ctypes
            SEM_NOGPFAULTERRORBOX = 0x0002 # From MSDN
            ctypes.windll.kernel32.SetErrorMode(SEM_NOGPFAULTERRORBOX);
            flags = CREATE_NO_WINDOW
            shell = not self._cmd[0].lower().endswith(".exe")
        else:
            flags = 0
            shell=False
        self.p = subprocess.Popen(self._cmd,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT,
                                  shell=shell,
                                  cwd=self._cwd,
                                  # universal_newlines=True,
                                  creationflags=flags)
        while True:
            line = self.p.stdout.readline()
            if not line:
                break
            print line,
        self.p.wait()

def main(test_dir, cmd, executable):
    test_dir = os.path.abspath(test_dir)
    harness_dir = os.path.abspath(os.path.dirname(__file__))
    php_harness_file = os.path.join(harness_dir, 'drive_testrunner.php')
    if not cmd and os.path.exists(php_harness_file):
        if not executable:
            executable = "php"
        cmd = [executable, php_harness_file]
    if cmd:
        # print "Running cmd [%s] in dir %s" % (cmd, test_dir)
        harness = KoHarnessPHPTests(cmd, test_dir)
        harness.run()
    else:
        # Report a failed test run in terms that sleuth will parse
        print "@suite_started@: failed attempt to run tests"
        print "@fault@: No command-line specified, and no Komodo/PHP test runner "
        print "@fault@: (drive_testrunner.php) found in directory " + test_dir
        print "@suite_finished@:"
        
if __name__ == "__main__":
    executable = None
    usage =  "usage: " + sys.argv[0] + "[ -x|--executable=interpreter-path] [cwd [command args...]]"
    import getopt
    optlist, args = getopt.getopt(sys.argv[1:], 'hx:', ['help', 'executable='])
    for opt, optarg in optlist:
        if opt in ('-x', '--executable'):
            executable = optarg
        elif opt in ('-h', '--help'):
            sys.stderr.write(usage + "\n")
            sys.exit()
    if len(args) == 0:
        cwd = os.getcwd()
        cmd = None
    else:
        cwd = args[0]
        os.chdir(cwd)
        if len(args) == 1:
            cmd = None
        else:
            cmd = args[1:]
    main(cwd, cmd, executable)
