#!python
# Copyright (c) 2007 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

"""Invoke a suite of Ruby tests, and parse the results.
"""

import re, sys, os

CREATE_NO_WINDOW = 0x8000000 # same as win32process.CREATE_NO_WINDOW

class KoHarnessRubyTests(object):
    def __init__(self, cmd="rake", cwd=None):
        self._cmd = cmd
        if cwd is None:
            self._cwd = os.getcwd()
        else:
            self._cwd = cwd
        
    def run(self):
        import subprocess
        if sys.platform.startswith("win"):
            # Don't display the Windows GPF dialog if the invoked program dies.
            # See comp.os.ms-windows.programmer.win32
            #  How to suppress crash notification dialog?, Jan 14,2004 - Raymond Chen's response
            # (URL http://groups.google.com/group/comp.os.ms-windows.programmer.win32/tree/browse_frm/thread/eeade5ecfa04d000/658b412a62ef5d7a?hl=en&rnum=1&q=suppressing+%22has+encountered+a+problem+and+needs+to+close%22+windows&_done=%2Fgroup%2Fcomp.os.ms-windows.programmer.win32%2Fbrowse_frm%2Fthread%2Feeade5ecfa04d000%2F74397b6bf9405304%3Fhl%3Den%26lnk%3Dst%26q%3Dsuppressing%2B%2522has%2Bencountered%2Ba%2Bproblem%2Band%2Bneeds%2Bto%2Bclose%2522%2Bwindows%26#doc_41fe71be34d3614b)
            import ctypes
            SEM_NOGPFAULTERRORBOX = 0x0002 # From MSDN
            ctypes.windll.kernel32.SetErrorMode(SEM_NOGPFAULTERRORBOX);
            flags = CREATE_NO_WINDOW
            shell = not self._cmd[0].lower().endswith(".exe")
        else:
            flags = 0
            shell=False
        #print "#QQQ: Ruby -- running %r in %s" % (self._cmd, self._cwd)
        self.p = subprocess.Popen(self._cmd,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT,
                                  shell=shell,
                                  cwd=self._cwd,
                                  # universal_newlines=True,
                                  creationflags=flags)
        target_cwd = self._cwd.lower()
        if sys.platform == "win32":
            target_cwd = target_cwd.replace("\\", "/")
        line_num = 0
        internal_stack_trace_re = re.compile(r'harnesses[\\/]ruby[\\/]load_komodo_runner\.rb\s*:\s*\d+\s*:\s*in')
        while True:
            line = self.p.stdout.readline()
            if not line:
                break
            line_num += 1
            # Pull out the starting Rake announcement line
            if line_num == 1:
                if (line.strip().lower().replace("\\", "/") ==
                    "(in " + target_cwd + ")"):
                    continue
            if internal_stack_trace_re.search(line):
                continue
            print line,
        self.p.wait()

def main(test_dir, cmd=None, executable=None):
    """ Most of the time tests will be run with rake, but
    the user will need to supply an argument.  Analyze the
    supplied command, and if it is a rake command, modify it
    so it loads the Komodo test runner."""
    rakefile = None
    test_dir = os.path.abspath(test_dir)
    rake_cmd = None
    rake_args = []
    if cmd and cmd[0].lower() == "rake":
        rake_cmd = cmd[0]
        rake_args = cmd[1:]
        cmd = None
    if not cmd:
        if os.path.exists(os.path.join(test_dir, "Rakefile")):
            rakefile = os.path.join(test_dir, "Rakefile")
        elif not sys.platform.startswith("win") and os.path.exists(os.path.join(test_dir, "rakefile")):
            rakefile = os.path.join(test_dir, "rakefile")
        if rakefile is not None:
            cmd = [rake_cmd or 'rake',
                   '--libdir=' + os.path.abspath(os.path.dirname(__file__)),
                   '--require=add_rake_runner',
                   '-f',
                   rakefile,
                   'TESTOPTS=--runner=komodo -v']
            if rake_args:
                cmd += rake_args
                
    if not cmd and rake_cmd:
        cmd = [rake_cmd] + rake_args
    
    if cmd:
        # print "#QQQ: Running cmd [%s] in dir %s" % (cmd, test_dir)
        harness = KoHarnessRubyTests(cmd, test_dir)
        harness.run()
    else:
        # Report a failed test run in terms that sleuth will parse
        print "@suite_started@: failed attempt to run tests"
        print "@fault@: No command-line specified, and no Rakefile found in "
        print "@fault@: directory " + test_dir
        print "@suite_finished@:"
        
if __name__ == "__main__":
    executable = None  # Currently not used
    usage =  "usage: " + sys.argv[0] + "[ -x|--executable=interpreter-path] [cwd [command args...]]"
    import getopt
    optlist, args = getopt.getopt(sys.argv[1:], 'hx:', ['help', 'executable='])
    for opt, optarg in optlist:
        if opt in ('-x', '--executable'):
            executable = optarg
        elif opt in ('-h', '--help'):
            sys.stderr.write(usage + "\n")
            sys.exit()
    if len(args) == 0:
        cwd = os.getcwd()
        cmd = None
    else:
        cwd = args[0]
        os.chdir(cwd)
        if len(args) == 1:
            cmd = None
        else:
            cmd = args[1:]
    main(cwd, cmd)
