#!/usr/bin/perl -w
use strict;
use IO::Handle;

my $real_cvs;
# Either load the existing CSV class, or define what we need here.
BEGIN {
    eval {
        require Text::CSV_XS;
        $real_cvs = 1;
    };
    if ($@) {
        $real_cvs = 0;
        # Define the class here
        package Text::CSV_XS::Fake;
        my $contents = qr/(?:\\.|[^"])*/;
        sub new {
            my ($cls) = @_;
            return bless {}, $cls;            
        }
        sub getline {
            my ($self, $in) = @_;
            my $in_string = 0;
            my $curr_cols = [];
            my $curr_col = \$curr_cols->[0];
            while (1) {
                # Keep reading input until we reach the end
                my $line = <$in>;
                return undef unless $line;
                chomp $line;
                if ($in_string) {
                    if ($line =~ m/^($contents)"(.*)/) {
                        $$curr_col .= $1;
                        $line = $2;
                        $in_string = 0;
                    } else {
                        $$curr_col .= $line . $/;
                        next;
                    }
                }
                while ($line) {
                    if ($line =~ /^,\s*(.*)/) {
                        push @$curr_cols, "";
                        $curr_col = \$curr_cols->[-1];
                        $line = $1;
                    } elsif ($line =~ /^\s+(.*)/) {
                        $line = $1;
                    } elsif ($line =~ /^"($contents)$/) {
                        $$curr_col .= $1 . $/;
                        $in_string = 1;
                        $line = "";
                    } elsif ($line =~ /^"($contents)"\s*(.*)/) {
                        $$curr_col .= $1;
                        $line = $2;
                    } elsif ($line =~ /^([^,]*)(.*)/) {
                        $$curr_col .= $1;
                        $line = $2;
                    }
                }
                last unless $in_string;
            }
            return $curr_cols;
        }
    }
}

open my $in,  "< mailexport.txt" or die "Cannot open mailexport.txt: $!";
open my $out, "> mailexport.xml" or die "Cannot write mailexport.xml: $!";

print $out <<EOT;
<?xml version="1.0"?>
<EMAILCOMMENTS>

EOT

# Process file in binary mode (fields may contain \n characters)
my $csv = $real_cvs ? Text::CSV_XS->new({binary => 1}) : Text::CSV_XS::Fake->new;

# First record contains list of fieldnames
my $fields = $csv->getline($in);
die "No input at all" unless $fields;
while (1) {
    my $record = $csv->getline($in);

    # getline() returns a ref to an empty array at the end of file
    last unless defined $record;
    next unless $record && @$record;

    # Encode "<" characters as "&lt;" and "&" as "&amp;" in all fields
    foreach (@$record) {
	s/&/&amp;/g;
	s/</&lt;/g;
    }

    # Create hash of fields using hash slice
    my %record;
    @record{@$fields} = @$record;

    print $out <<EOT;
    <EMAIL>

	<HEADER>
	    <ORIGADDRESS>$record{"From: (Address)"}</ORIGADDRESS>
	    <DESTADDRESS>$record{"To: (Address)"}</DESTADDRESS>
	    <SUBJECT>$record{"Subject"}</SUBJECT>
	</HEADER>

	<BODY>$record{"Body"}</BODY>

    </EMAIL>

EOT
}
print $out "</EMAILCOMMENTS>\n";
close $out or die "Can't write mailexport.xml: $!";
close $in;
