/* Copyright (c) 2000-2007 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

//XXX Add _g prefixes to mark these as module-globals

//TODO: Merge this into pref-project-sleuth-properties.js, 
// based on how it's init'ed

var sleuth_dirname, sleuth_command_line, sleuth_language, ko;
// No need to define ko for this page
var languageIndices = {};

var current_testplan_name = null;
var current_testplan_pref = null;
var default_testplan_name_count = 0;
var default_lang = null;
var project;

// The copy of the prefs we work with here.
var gSleuthPrefs;

const DEFAULT_TESTPLAN_NAME = 'New test plan';
const TESTPLAN_PREFNAME = 'testPlans';
const LAST_TESTPLAN_IN_PREF_PREFNAME = 'lastTestPlanName_pref';

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var names_menulist_id;
var names_popup_menu_id;
var get_default_dir;

function PrefSleuth_OnLoad()
{
    ko = parent.opener.ko;
    names_menulist_id = "prefs-global-sleuth-tp-names";
    names_popup_menu_id = "prefs-global-sleuth-tp-names-popup";
    project = null;
    get_default_dir = function() {
        return ko.window.getCwd();
    };
    parent.hPrefWindow.onpageload();
}

function PrefSleuth_Project_OnLoad()
{
    ko = parent.opener.ko;
    project = ko.projects.manager.getSelectedProject();
    names_menulist_id = "prefs-project-sleuth-tp-names";
    names_popup_menu_id = "prefs-project-sleuth-tp-names-popup"
    get_default_dir = function() {
        return project.getFile().dirName;
    };
    parent.hPrefWindow.onpageload();
}

function OnPreferencePageInitalize(prefset)
{
    gSleuthPrefs = null;
    // Populate the list of test-plan names, and determine which
    // test-plan we'll set up.
    sleuth_dirname = document.getElementById('sleuth_dirname');
    sleuth_command_line = document.getElementById('sleuth_command_line');
    sleuth_language = document.getElementById('sleuth_language');
    var menuitems = sleuth_language.getElementsByTagName('menuitem');
    for (var i = 0; i < menuitems.length; i++) {
        languageIndices[menuitems[i].label] = i;
    }
}

function OnPreferencePageLoading(prefset)
{
    if (prefset.hasPrefHere(TESTPLAN_PREFNAME)) {
        gSleuthPrefs = prefset.getPref(TESTPLAN_PREFNAME);
    }
    if (!gSleuthPrefs) {
        gSleuthPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    }
    current_testplan_pref = null;
    try {
        if (prefset.hasStringPref(LAST_TESTPLAN_IN_PREF_PREFNAME)) {
            current_testplan_name = prefset.getStringPref(LAST_TESTPLAN_IN_PREF_PREFNAME);
        } else {
            current_testplan_name = null;
        }
        // Take the first, if there is one
        var testplan_names = {};
        var countObj = {};
        gSleuthPrefs.getPrefIds(testplan_names, countObj);

        if (!current_testplan_name && countObj.value > 0) {
            current_testplan_name = testplan_names.value[0];
            // set the pref when we build the menu
        }
    
        // Fill up the test plan-name menu, and find the current test plan
        var menupopup = document.getElementById(names_popup_menu_id);
        _clearPopupMenu(menupopup);
        for (var i = 0; i < countObj.value; i++) {
            var tp_name = testplan_names.value[i];
            _addPopupMenuItem(menupopup, tp_name);
            if (!current_testplan_pref && tp_name == current_testplan_name) {
                current_testplan_pref = gSleuthPrefs.getPref(tp_name);
                _fillTestPlan(current_testplan_pref, current_testplan_name);
            }
        }
        if (!current_testplan_pref) {
            if (countObj.value > 0) {
                current_testplan_name = testplan_names.value[0];
                current_testplan_pref = gSleuthPrefs.getPref(current_testplan_name);
                _fillTestPlan(current_testplan_pref, current_testplan_name);
            } else {
                // Create a default test plan and pref
                _createInstallDefaultTestPlan();
            }
        } else {
            document.getElementById(names_menulist_id).setAttribute('label', current_testplan_name);
        }
    } catch(e) {
        log.exception(e);
    }
}

function OnPreferencePageClosing(prefset, ok) {
    //   called when the user clicks ok
    if (!ok) return true;
    try {
        if (!fieldsCheck()) {
            //Note: the dialog still closes.
            return false;
        }
        if (current_testplan_name) {
            savePrefs(current_testplan_pref);
            gSleuthPrefs.setPref(current_testplan_name, current_testplan_pref);
            prefset.setStringPref(LAST_TESTPLAN_IN_PREF_PREFNAME,
                                  current_testplan_name);
        }
        
        if (prefset.hasPrefHere(TESTPLAN_PREFNAME)) {
            prefset.deletePref(TESTPLAN_PREFNAME);
        }
        prefset.setPref(TESTPLAN_PREFNAME, gSleuthPrefs);
        // This doesn't chain into the project prefs notification service,
        // so trigger it manually.
        if (!project || ko.projects.manager.getCurrentProject() == project) {
            try {
                var obSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
                obSvc.notifyObservers(this, TESTPLAN_PREFNAME, project ? project.url : "<global>");
            } catch(ex) {
                log.exception(ex);
            }
        }
    } catch(e) {
        log.exception(e);
    }
    return true;
}

// OnPreferencePageCancel(prefset)
//      called when the user clicks cancel


function _clearPopupMenu(menupopup) {
    while (menupopup.childNodes.length) {
        menupopup.removeChild(menupopup.lastChild);
    }
}

function _addPopupMenuItem(menupopup, label) {
    var menuitem = document.createElementNS(XUL_NS, 'menuitem');
    menuitem.setAttribute('label', label);
    menupopup.appendChild(menuitem);
}

function _createInstallDefaultTestPlan() {
    // Rebuild the menu
    var this_testplan_name = _getUniqueDefaultTestPlanName();
    default_testplan_name_count += 1;
    var menupopup = document.getElementById(names_popup_menu_id);
    _clearPopupMenu(menupopup);
    _addPopupMenuItem(menupopup, this_testplan_name);
    
    current_testplan_name = this_testplan_name;
    current_testplan_pref = newPrefs();
    gSleuthPrefs.setPref(current_testplan_name, current_testplan_pref);
    _fillTestPlan(current_testplan_pref, current_testplan_name);
}

function _fillTestPlan(testPlanPref, name) {
    // Fill in the current details
    document.getElementById(names_menulist_id).setAttribute('label', name);
    _fillTestPlanDetails(testPlanPref);
}

function _fillTestPlanDetails(testPlanPref) {
    sleuth_dirname.value = ko.sleuth.manager.dirFromPrefs(testPlanPref);
    sleuth_command_line.value = testPlanPref.getStringPref("command_line");
    var language = testPlanPref.getStringPref("language");
    if (language in languageIndices) {
        sleuth_language.selectedIndex = languageIndices[language];
    } else {
        sleuth_language.selectedIndex = 0;
    }
}

function _getNumTestPlans() {
    var countObj = {};
    gSleuthPrefs.getPrefIds({}, countObj);
    return countObj.value;
}

// The caller of this function has to increment the default-count,
// in case the user cancels out on the suggested name.
function _getUniqueDefaultTestPlanName() {
    var test_plan_count = _getNumTestPlans();
    if (default_testplan_name_count <= test_plan_count) {
        default_testplan_name_count = test_plan_count + 1;
    }
    return (DEFAULT_TESTPLAN_NAME
            + " #"
            + default_testplan_name_count);
}

function doCreateNewTestPlan() {
    // first, get a name for the new test plan
    try {
        if (!fieldsCheck()) {
            return;
        }
        var name = ko.dialogs.prompt("Enter a new test plan name.  The new "+
                                     "test plan will be based on the current one ",
                                     "Test Plan Name",
                                     _getUniqueDefaultTestPlanName(),
                                     "Test Plan Name");
        if (!name) return;
        default_testplan_name_count += 1;
        if (gSleuthPrefs.hasPref(name)) {
            ko.dialogs.alert("A test plan with the name '"+name+"' already exists.\n");
            return;
        }
        createNewPrefSetFromCurrent(name);
        var menupopup = document.getElementById(names_popup_menu_id);
        var menuitem = document.createElementNS(XUL_NS, 'menuitem');
        menuitem.setAttribute('label', name);
        menupopup.appendChild(menuitem);
        document.getElementById(names_menulist_id).setAttribute('label', name);
        // XXX This button should never be disabled.
        //document.getElementById('deleteButton').removeAttribute('disabled');
    } catch(e) {
        log.exception(e);
    }
}

function doDeleteTestPlan() {
    // Remove the test plan from the prefs,
    // and then remove it from the popup menu, and substitute
    // either the one before
    // it in the menupopup menu, the second if it's the first,
    // or put up the default menu again.
    
    gSleuthPrefs.deletePref(current_testplan_name);
    var menupopup = document.getElementById(names_popup_menu_id);
    var menuitems = menupopup.getElementsByTagNameNS(XUL_NS, 'menuitem');
    var new_name = null;
    for (var i = 0; i < menuitems.length; i++) {
        if (menuitems[i].getAttribute('label') == current_testplan_name) {
            if (i == 0) {
                if (menuitems.length == 1) {
                    _createInstallDefaultTestPlan();
                    return;
                } else {
                    new_name = menuitems[i+1].getAttribute("label");
                }
            } else {
                new_name = menuitems[i-1].getAttribute("label");
            }
            menupopup.removeChild(menuitems[i]);
            break;
        }
    }
    if (new_name) {
        current_testplan_name = new_name;
        current_testplan_pref = gSleuthPrefs.getPref(current_testplan_name);
        _fillTestPlan(current_testplan_pref, current_testplan_name);
    } else {
        dump("Couldn't find a test plan named " + current_testplan_name
             + "\n");
        current_testplan_name = null;
    }
}

function newPrefs() {
    var new_pref = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    ko.sleuth.manager.setPrefToDir(new_pref, get_default_dir());
    new_pref.setStringPref('command_line', '');
    var supported_languages = {};
    var supported_language_items = sleuth_language.getElementsByTagName('menuitem');
    for (var i = 0; i < supported_language_items.length; i++) {
        supported_languages[supported_language_items[i].label] = i;
    }
    if (!project) {
        // If there's a file, see if its language is in our list
        try {
            var current_lang = ko.views.manager.currentView.document.language;
            var found = current_lang in supported_languages;
            new_pref.setStringPref('language',
                                   found ? current_lang :
                                   (supported_language_items.length > 0 ?
                                    supported_language_items[0].label : ""));
        } catch (ex) {
            dump(ex + "\n");
            new_pref.setStringPref('language', 'Perl'); // it's not going away
        }
    } else {
        if (!default_lang) {
            var possible_lang = (ko.sleuth.manager.getFirstRecognizedProjectLanguage(project, supported_languages)
                                 || sleuth_language.getElementsByTagName('menuitem')[0].label
                                 || "Perl");
            default_lang = possible_lang;
        }
        new_pref.setStringPref('language', default_lang);
    }
    return new_pref;
}

function createNewPrefSetFromCurrent(name) {
    // save any settings in the current prefs
    var oldPrefs = savePrefs(current_testplan_pref);
    current_testplan_pref = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    var prefNames = {'directory':null,
                     'command_line':null,
                     'language':null};
    for (var prefName in prefNames) {
        // copying the raw URI is fine, since it isn't being used here
        current_testplan_pref.setStringPref(prefName,
                                            oldPrefs.getStringPref(prefName));
    }
    current_testplan_name = name;
    gSleuthPrefs.setPref(name, current_testplan_pref);
}

function _getLanguage(menulist) {
    return menulist.getAttribute("label");
}

function fieldsAreOK() {
    if (sleuth_dirname.value) {
        return {result:true};
    } else {
        return {result : false,
                message: "Please specify a directory to run the tests from",
                field  : sleuth_dirname};
    }
}

function savePrefs(prefs) {
    var sleuth_language_value = _getLanguage(sleuth_language);
    ko.sleuth.manager.setPrefToDir(prefs, sleuth_dirname.value);
    prefs.setStringPref("command_line", sleuth_command_line.value);
    prefs.setStringPref("language", sleuth_language_value);
    return prefs;
}

function fieldsCheck() {
    var status = fieldsAreOK();
    if (!status.result) {
        if (status.message) {
            alert(status.message);
        }
        document.getElementById(names_menulist_id).setAttribute('label', current_testplan_name);
        if (status.field) {
            status.field.focus();
        }
        return false;
    }
    return true;
}

function ChangeTestPlan() {
    if (!fieldsCheck()) {
        return;
    }
    var old_name = current_testplan_name;
    var old_prefs = savePrefs(current_testplan_pref);
    current_testplan_name = document.getElementById(names_menulist_id).getAttribute('label');
    current_testplan_pref = gSleuthPrefs.getPref(current_testplan_name);
    _fillTestPlanDetails(current_testplan_pref);
}


function _getFirstRecognizedLanguage(part) {
}

function doBrowseForDir() {
    var dir = ko.filepicker.getFolder(sleuth_dirname.value);
    if (dir) sleuth_dirname.value = dir;
};

