/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// A project extension to manage source code control operations in Komodo.

if (typeof(ko)=='undefined') {
    var ko = {};
}
if (typeof(ko.projects)=='undefined') {
    ko.projects = {};
}

(function() {

var log = ko.logging.getLogger('peSCC');
var _prefs = Components.classes["@activestate.com/koPrefService;1"].
                    getService(Components.interfaces.koIPrefService).prefs;
var _obSvc = Components.classes["@mozilla.org/observer-service;1"].
             getService(Components.interfaces.nsIObserverService);

//---- SCC project extension implementation.

function peSCC() {
    this.name = 'SCC';
    //log.setLevel(ko.logging.LOG_DEBUG);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peSCC.prototype.constructor = peSCC;

peSCC.prototype.init = function() {
    // register our command handlers
    this.registerCommands();

    // this adds columns to the project/toolbox tree's
    var em = ko.projects.extensionManager;
    em.setDatapoint('Status','sccStatus');
    em.setDatapoint('Rev','sccLocalRevision');
    em.setDatapoint('Depot Rev','sccDepotRevision');
    em.setDatapoint('Action','sccAction');
}

peSCC.prototype.registerCommands = function() {
    var em = ko.projects.extensionManager;
    em.registerCommand('cmd_SCCedit', this);
    em.registerCommand('cmd_SCCadd', this);
    em.registerCommand('cmd_SCCremove', this);
    em.registerCommand('cmd_SCCupdate', this);
    em.registerCommand('cmd_SCCcommit', this);
    em.registerCommand('cmd_SCCdiff', this);
    em.registerCommand('cmd_SCChistory', this);
    em.registerCommand('cmd_SCCrevert', this);

    em.registerCommand('cmd_SCCadd_folder', this);
    em.registerCommand('cmd_SCCupdate_folder', this);
    em.registerCommand('cmd_SCCcommit_folder', this);
    em.registerCommand('cmd_SCCdiff_folder', this);
    em.registerCommand('cmd_SCCrevert_folder', this);
}

peSCC.prototype.registerEventHandlers = function() {
}

peSCC.prototype.registerMenus = function() {
    var menupopup = document.getElementById('menu_SCCmenu_context');
    if (!menupopup) {
        log.error("Couldn't get 'menu_SCCmenu'");
        return;
    }
    var folderpopup = document.getElementById('menu_SCCmenu_folder_context');
    if (!folderpopup) {
        log.error("Couldn't get 'menu_SCCmenu_folder'");
        return;
    }
    var em = ko.projects.extensionManager;
    em.addMenuItem(Components.interfaces.koIPart_SCC_file, document.createElement('menuseparator'));
    em.addMenuItem(Components.interfaces.koIPart_SCC_file, menupopup);
    em.addMenuItem(Components.interfaces.koIPart_SCC_collection, document.createElement('menuseparator'));
    em.addMenuItem(Components.interfaces.koIPart_SCC_collection, folderpopup);
    em.addMenuItem(Components.interfaces.koIPart_SCC_directory, document.createElement('menuseparator'));
    em.addMenuItem(Components.interfaces.koIPart_SCC_directory, folderpopup);
}

/* This takes an item, and determines what type of scc
   flags we are dealing with. returns true if we can do anything, or
   false if we cannot do anything */
peSCC.prototype.determineItemSCCSupport = function(/* koIPart */item, sccObj) {
    // prevent non-cvs directories from having to go through this torture
    var file = item.getFile();
    if (file.sccExclude) {
        return false;
    }
    // assume the worst =)
    sccObj.isscc_file = false;
    sccObj.isscc_directory = false;
    sccObj.isscc_container = false;
    sccObj.isscc_collection = false;

    var ok;
    try {
        ok = item.QueryInterface(Components.interfaces.koIPart_SCC_file) != null;
        if (ok) {
            sccObj.isscc_file = true;
        }
    } catch(e) {}
    try {
        ok = item.QueryInterface(Components.interfaces.koIPart_SCC_collection) != null;
        if (ok) {
            sccObj.isscc_collection = true;
        }
    } catch(e) {}
    try {
        ok = item.QueryInterface(Components.interfaces.koIPart_SCC_directory) != null;
        if (ok) {
            sccObj.isscc_directory = true;
        }
    } catch(e) {}
    sccObj.isscc_container = sccObj.isscc_directory || sccObj.isscc_collection;
    sccObj.isscc_file = sccObj.isscc_file || !sccObj.isscc_container;
    sccObj.has_scc = file.sccType != '';
    sccObj.is_scc = sccObj.isscc_file || sccObj.isscc_container;
    if (!sccObj.is_scc) {
        return false;
    }

    sccObj.can_scc = sccObj.has_scc || file.sccDirType != '';
    sccObj.missing = !file.exists;

    var in_repository = null;
    if (item.url && !sccObj.can_scc && !sccObj.missing) {
        sccObj.can_scc = sccObj.has_scc || file.sccDirType;
    }

    if (file.sccExclude) return false;
    return true;
}

peSCC.prototype.supportsCommand = function(command, item) {
    // do we have either cvs or scc?
    if (!_prefs.getBooleanPref('cvsEnabled') &&
        !_prefs.getBooleanPref('p4Enabled') &&
        !_prefs.getBooleanPref('svnEnabled')) {
        return false;
    }
    // If scintilla has focus, should be talking about the file in scintilla
    if (!item) {
        if (ko.projects.getFocusedProjectView()) {
            item = ko.projects.active.getSelectedItem();
            // item must be a file
            if (!item || !item.url) {
                return false;
            }
        } else {
            var view = ko.views.manager.currentView;
            if (!view || !view.document || view.document.isUntitled) {
                // This is an unsaved file -- no SCC commands can apply.
                return false;
            }
            item = view.item;
            item.url = view.document.file.URI;
        }
    }
    // XXX look into cacheing sccObj into the item
    var sccObj = new Object();
    if (!this.determineItemSCCSupport(item,sccObj)) {
        return false;
    }
    var isfoldercmd = command.indexOf('_folder')>1;
    /* if cvs, and this is a directory, then the directory MUST be in cvs
    if (isfoldercmd) {
      dump("URL: "+item.url+"\n");
      ko.logging.dumpObject(sccObj);
      dump("  isfoldercmd "+isfoldercmd+"\n");
      dump("  folders_in_repository "+folders_in_repository+"\n");
    }
    */
    if (isfoldercmd && !sccObj.isscc_container) {
        return false;
    }
    /* folder handling is different between p4 and cvs */
    var file = item.getFile();
    var folders_in_repository = file.sccType == 'cvs' || file.sccDirType == 'cvs' ||
                        file.sccType == 'svn' || file.sccDirType == 'svn';
    var folders_revertable = file.sccType == 'p4' || file.sccDirType == 'p4';

    var foldercmd_ok = (sccObj.isscc_container && isfoldercmd) &&
            ((folders_in_repository && (sccObj.isscc_directory || sccObj.isscc_collection)) ||
            (!folders_in_repository && sccObj.can_scc));
    var retval;
    switch (command) {
    case 'cmd_SCCedit':
        retval = sccObj.has_scc &&
            (file.sccAction == '' || file.sccAction == 'integrate')
            && (file.sccType == 'p4' ||
                file.sccType == 'cvs' && file.isReadOnly)
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCadd_folder':
        retval = sccObj.can_scc && !sccObj.has_scc && isfoldercmd
                && sccObj.isscc_directory && folders_in_repository;
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCadd':
        retval = sccObj.can_scc && !sccObj.has_scc;
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCupdate_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCCupdate':
        retval = sccObj.has_scc && file.sccNeedSync;
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCremove':
        return sccObj.has_scc && !file.sccNeedSync &&
            file.sccAction == '';
        break;
    case 'cmd_SCCdiff_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCChistory':
        retval = sccObj.has_scc;
        return retval;
        break;
    case 'cmd_SCCdiff':
        retval = sccObj.has_scc &&(file.sccNeedSync ||
                file.sccAction == 'edit' || file.sccAction == 'integrate');
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCrevert_folder':
        return foldercmd_ok && folders_revertable;
        break;
    case 'cmd_SCCrevert':
        retval = sccObj.has_scc && (file.sccNeedSync ||
                file.sccAction != '');
        //dump('asked about '+ command + ', returning' + retval +'\n');
        return retval;
        break;
    case 'cmd_SCCcommit_folder':
        return foldercmd_ok;
        break;
    case 'cmd_SCCcommit':
        return sccObj.has_scc && file.sccAction != ''
            && (!file.sccNeedSync ||
                file.sccAction == 'delete');
        break;
    default:
        break;
    }
    return false;
}

peSCC.prototype.isCommandEnabled = peSCC.prototype.supportsCommand;

/* recursivly build a list of items that are being used in
   this scc operation */
peSCC.prototype._addVirtualFolderChildren = function(item)
{
    var items = [];
    try {
        var children = new Array();
        item.getChildren(children, new Object());
        children = children.value;
    } catch(e) {
        return items;
    }
    if (children.length < 1) return items;

    var child;
    for (var i = 0; i < children.length; i++) {
        child = children[i];
        if (child.url) {
            items.push(child);
        } else {
            var c = this._addVirtualFolderChildren(child);
            if (c.length > 0) {
                items = items.concat(c);
            }
        }
    }
    return items;
}

peSCC.prototype.doCommand = function(command) {
    window.setTimeout(function (self, command) { self._doCommand(command); }, 1, this, command);
}

peSCC.prototype._doCommand = function(command) {
    var cmd = null;
    var error = null;
    var urls=[];
    var _sccUrls = {};
    var _sccLocalVersions = {}; /* Only used for scc history */
    var _allURLS = [];
    var file;
    var item = null;
    var items = [];
    var i;
    var sccType;

    log.debug("doCommand "+command);

    // Iterate over all the targeted items (project items, files, ...) and
    // gather the urls we are going to perform SCC actions upon

    // XXX FIXME this is an issue since projects do not necessarily have focus
    // on a context menu operation
    if (ko.projects.getFocusedProjectView()) {
        // Get the items from the focused project
        items = ko.projects.active.getSelectedItems();
    } else {
        // Get the items from the currently focused file
        //dump("Scintilla has focus -- actions should be based on the current file\n");
        var view = ko.views.manager.currentView;
        if (view && !view.document.isUntitled) {
            item = ko.projects.manager.findItemByURL(view.document.file.URI);
            if (!item) {
                item = view.item;
                item.url = view.document.file.URI;
                item.project = null;
            }
            items.push(item);
        }
    }

    if (items.length < 1) return;

    // We have items, now work out the urls for the scc handlers
    var isfoldercmd = command.indexOf('_folder')>1;
    for (i=0; i<items.length; i++) {
        item = items[i];
        file = item.getFile();
        sccType = file.sccType || file.sccDirType;
        if (!sccType || file.sccExclude) {
            log.debug("doCommand unable to determineItemSCCSupport "+item.url);
            continue
        }

        if (typeof(_sccUrls[sccType]) == 'undefined') {
            _sccUrls[sccType] = [];
            _sccLocalVersions[sccType] = [];
        }
        if (isfoldercmd && item.type == 'project') {
            var url = ko.uriparse.dirName(item.url)
            _sccUrls[sccType].push(url);
            _sccLocalVersions[sccType].push(""); /* No version, it's a collection */
            _allURLS.push(url);
            // add the dialogs children
            //items = items.concat(this._addVirtualFolderChildren(item));
        } else {
            _sccUrls[sccType].push(item.url);
            _sccLocalVersions[sccType].push(file.sccLocalRevision);
            _allURLS.push(item.url);
        }

        // XXX implement virtual folder support
        // check for virtual folders, add the children of the virtual
        // folders to our items array.
        //if (sccObj.isscc_collection && items.children.length > 0) {
        //    var c = this._addVirtualFolderChildren(item);
        //    if (c.length > 0) {
        //        items = items.concat(c);
        //    }
        //}
    }
    
    if ((command != 'cmd_SCCrevert') &&
        (command != 'cmd_SCChistory') &&
	!ko.views.manager.offerToSave(_allURLS, /* urls */
                      'Save Modified Files?', /* title */
                      'Save Modified Files Before Source Code Control Action', /* prompt */
                      "save_on_scc", /* pref */
                      false /* skip projects */
                      )) {
        return;
    }
    
    // We've found all the SCC urls, now perform the scc actions for these urls.

    var output = '';
    var title;
    var sccType;
    var sccSvc;

    log.debug('selected for scc operation '+_sccUrls +'\n');
    for (sccType in _sccUrls) {
        urls = _sccUrls[sccType];
        if (urls.length <= 0)
            continue;

        // Get the koISCC service object
        var cid = "@activestate.com/koSCC?type=" + sccType + ";1";
        sccSvc = Components.classes[cid].getService(Components.interfaces.koISCC);
        if (!sccSvc || !sccSvc.isFunctional)
            continue;

        try {
            switch (command) {
            case 'cmd_SCCedit':
                ko.scc.Edit(sccSvc, urls);
                break;
            case 'cmd_SCCadd_folder':
                if (!isfoldercmd) return;
                // else fall through
            case 'cmd_SCCadd':
                ko.scc.Add(sccSvc, urls);
                break;
            case 'cmd_SCCremove':
                ko.scc.Remove(sccSvc, urls);
                break;
            case 'cmd_SCCrevert_folder':
                if (!isfoldercmd) return;
                // else fall through
            case 'cmd_SCCrevert':
                ko.scc.Revert(sccSvc, urls);
                break;
            case 'cmd_SCCupdate_folder':
                if (!isfoldercmd) return;
                // else fall through
            case 'cmd_SCCupdate':
                ko.scc.Update(sccSvc, urls);
                break;
            case 'cmd_SCCdiff_folder':
                if (!isfoldercmd) return;
                // else fall through
            case 'cmd_SCCdiff':
                ko.scc.Diff(sccSvc, urls);
                break;
            case 'cmd_SCCcommit_folder':
                if (!isfoldercmd) return;
                // else fall through
            case 'cmd_SCCcommit':
                ko.scc.Commit(sccSvc, urls);
                break;
            case 'cmd_SCChistory':
                ko.scc.History(sccSvc, urls, _sccLocalVersions[sccType]);
                break;
            default:
                break;
            }
        } catch (e) {
            alert("An exception occurred during SCC operation. (" + e + ')');
            log.error(e);
        }
    }
    // Ensure the cached information gets updated, so the SCC toolbars and menu
    // are properly set.
    // Fixes bug: http://bugs.activestate.com/show_bug.cgi?id=48417
    ko.views.manager.resetLastViewCache();

    window.updateCommands('SCC'); // We've changed things hopefully, and no one else knows.
}

// this is hidden away now, no namespce, the registration keeps the reference
// we need
ko.projects.registerExtension(new peSCC());

}).apply(ko.projects);
