/* Copyright (c) 2000-2008 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

var ko, project;
// fields

var sleuth_name;
var sleuth_language;
var sleuth_dirname;
var sleuth_command_line;
// This line maps field name to required status

var text_field_names = {sleuth_name:[1, 0],
                        sleuth_dirname:[1, 2],
                        sleuth_command_line:[0, 3]};
var dialog_fields = {};

function doBrowseForDir() {
    // This is a bit coarse -- better to suggest either the
    // current dir, or if there is none, the project's; and
    // then if they cancel out don't put anything in the field.
    // The problem here is we'll start in Komodo's default directory.
    ko.filepicker.browseForDir(dialog_fields.sleuth_dirname);
    return true;
}

function onCancel() {
    return true;
}

function onOK() {
    var labels = document.getElementsByTagName("label");
    var missing_field = null;
    var missing_names = [];
    for (var name in text_field_names) {
        // labels[text_field_names[name][1]].style.color = 'black';
        if (text_field_names[name][0] && !dialog_fields[name].value) {
            var label;
            var labelNode = document.getElementById(dialog_fields[name] + "_label");
            if (labelNode) {
                label = labelNode.getAttribute("label");
            } else {
                label = name;
            }
            missing_names.push(label);
            if (!missing_field) {
                missing_field = dialog_fields[name];
            }
            //labels[text_field_names[name][1]].style.color = 'red';
        }
    }
    if (missing_names.length > 0) {
        alert("The following fields cannot be left blank: "
              + missing_names.join(", "));
        missing_field.focus();
        return false;
    }
    
    var gSleuthPrefs = null;
    var globalPrefs = Components.classes["@activestate.com/koPrefService;1"].getService(Components.interfaces.koIPrefService).prefs;
    var prefset = (project
                   ? project.prefset
                   : globalPrefs);
    const TESTPLAN_PREFNAME = 'testPlans';
    if (prefset.hasPrefHere(TESTPLAN_PREFNAME)) {
        gSleuthPrefs = prefset.getPref(TESTPLAN_PREFNAME);
    } else {
        gSleuthPrefs = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
        prefset.setPref(TESTPLAN_PREFNAME, gSleuthPrefs);
    }

    var name = dialog_fields['sleuth_name'].value;
    if (gSleuthPrefs.hasPrefHere(name)) {
        var owner = (project
                     ? ("The current project \""
                        + project.name
                        + "\"")
                     : "The global preference-set");
        alert(owner
              + " already has a test plan named \""
              + name
              + "\"");
        return false;
    }
    
    var testPlanPref = Components.classes["@activestate.com/koPreferenceSet;1"].createInstance();
    testPlanPref.setStringPref('command_line',
                               dialog_fields.sleuth_command_line.value);
    ko.sleuth.manager.setPrefToDir(testPlanPref,
                                   dialog_fields.sleuth_dirname.value);
    testPlanPref.setStringPref('language',
                               document.getElementById("sleuth_language").label);
    gSleuthPrefs.setPref(name, testPlanPref);
    // This doesn't chain into the project prefs notification service,
    // so trigger it manually.
    // Bug 74901: select this test-plan
    if (project && ko.projects.manager.getCurrentProject() != project) {
        ko.projects.manager.setCurrentProject(project);
    }
    try {
        var topic = project ? "select_project_testplan" : "select_global_testplan";
        globalPrefs.prefObserverService.notifyObservers(this, topic, name);
    } catch(ex) {}
    return true;
}

function onLoad() {
    ko = window.arguments[0];
    project = window.arguments[1];
    for (var name in text_field_names) {
        dialog_fields[name] = document.getElementById(name);
    }
    if (project) {
        dialog_fields.sleuth_name.value = (project.name
                                           + " - test plan # "
                                           + (ko.sleuth.manager.getNumProjectTestPlans(project) + 1));
        dialog_fields.sleuth_dirname.value = project.getFile().dirName;
    } else {
        dialog_fields.sleuth_name.value = ("global test plan # "
                                           + (ko.sleuth.manager.getNumGlobalTestPlans() + 1));
        try {
            dialog_fields.sleuth_dirname.value = ko.window.getCwd();
        } catch(ex) {
            // This shouldn't fail
            dump(ex + "\n");
            dialog_fields.sleuth_dirname.value = "";
        }
    }
    // Figure out the default language
    var current_lang;
    try {
        current_lang = ko.views.manager.currentView.document.language;
    } catch(ex) {
        current_lang = null;
    }
    var supported_languages = {};
    var supported_language_items = document.getElementsByTagName('menuitem');
    for (var i = 0; i < supported_language_items.length; i++) {
        supported_languages[supported_language_items[i].label] = i;
    }
    var idx = 0; // default: use the first entry in the menu (Perl)
    if (current_lang in supported_languages) {
        idx = supported_languages[current_lang];
    } else if (project) {
        current_lang = ko.sleuth.manager.getFirstRecognizedProjectLanguage(project, supported_languages);
        if (current_lang && current_lang in supported_languages) {
            idx = supported_languages[current_lang];
        }
    }
    var elt = document.getElementById("sleuth_language");
    elt.selectedIndex = idx;
}
