/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Sleuth main component
 *
 * Manages Test Plan Containers
 */

if (typeof(ko)=='undefined') {
    var ko = {};
}
if (!('sleuth' in ko)) {
    ko.sleuth = {}; // A namespace
}
(function() {

var sleuthLog = ko.logging.getLogger('sleuth');
sleuthLog.setLevel(ko.logging.LOG_DEBUG);

var Sleuth = function() {}; 
ko.sleuth.manager = new Sleuth(); // The sleuth singleton

Sleuth.prototype.TESTPLAN_PREFNAME = 'testPlans';

Sleuth.prototype.createNewTestPlan = function sl_createNewTestPlan(projectURL) {
    var project = (projectURL
                   ? ko.projects.manager.getProjectByURL(projectURL) : null);
    ko.windowManager.openOrFocusDialog(
        "chrome://komodo/content/sleuth/createNewTestPlan.xul",
        'komodo_tpcp',
        "chrome,modal,resizable,close=yes",
        ko, project);
}

Sleuth.prototype.createNewGlobalTestPlan = function sl_createNewGlobalTestPlan() {
    ko.windowManager.openOrFocusDialog(
        "chrome://komodo/content/sleuth/createNewTestPlan.xul",
        'komodo_tpcp',
        // "chrome,resizable,close=yes",
        "chrome,modal,resizable,close=yes",
        ko, null);
}

Sleuth.prototype._getGlobalTestPlanNames = function _getGlobalTestPlanNames(sorted) {
    if (typeof(sorted) == "undefined") {
        sorted = false;
    }
    var prefset = this._getGlobalPrefs();
    if (!prefset.hasPref(this.TESTPLAN_PREFNAME)) {
        return [];
    }
    return this.getTestPlanNamesFromPrefs(prefset.getPref(this.TESTPLAN_PREFNAME),
                                           sorted);
};

Sleuth.prototype._getProjectTestPlanNames = function _getProjectTestPlanNames(project, sorted) {
    if (typeof(sorted) == "undefined") {
        sorted = false;
    }
    var prefset = project.prefset;
    if (!prefset.hasPrefHere(this.TESTPLAN_PREFNAME)) {
        return [];
    }
    return this.getTestPlanNamesFromPrefs(prefset.getPref(this.TESTPLAN_PREFNAME),
                                           sorted);
};

Sleuth.prototype.getTestPlanNamesFromPrefs = function getTestPlanNamesFromPrefs(sleuthPrefs, sorted) {
    var testplan_names = {};
    sleuthPrefs.getPrefIds(testplan_names, {});
    var names = testplan_names.value;
    if (sorted) {
        names.sort(function(a, b) {
                a = a.toLowerCase();
                b = b.toLowerCase();
                return (a < b ? -1 :
                        a > b ? 1 : 0);
            });
    }
    return names;
};

Sleuth.prototype._getGlobalPrefs = function _getGlobalPrefs() {
    if (!this._globalPrefSvc) {
        this._globalPrefSvc = Components.classes["@activestate.com/koPrefService;1"].getService(Components.interfaces.koIPrefService);
    }
    return this._globalPrefSvc.prefs;
};

Sleuth.prototype.getNumProjectTestPlans = function getNumProjectTestPlans(project) {
    return this._getProjectTestPlanNames(project).length;
}

Sleuth.prototype.getNumGlobalTestPlans = function getNumGlobalTestPlans() {
    return this._getGlobalTestPlanNames().length;
}

Sleuth.prototype.updateGlobalTestPlanMenu = function sl_updateGlobalTestPlanMenu(popupmenu) {
    var testPlans = this._getGlobalTestPlanNames(true);
    this._fillPopupMenu(popupmenu, testPlans, "doSelectedGlobalTestPlan", "");
};

Sleuth.prototype.updateProjectTestPlanMenu = function sl_updateProjectTestPlanMenu(popupmenu) {
    var project = ko.projects.manager.getSelectedProject();
    var testPlans = this._getProjectTestPlanNames(project, true);
    this._fillPopupMenu(popupmenu, testPlans, "doSelectedProjectTestPlan", project.url);
}

Sleuth.prototype._fillPopupMenu = function sl_updateProjectTestPlanMenu(popupmenu, testPlans, methodName, projectURL) {
    // First clear the popupmenu
    while (popupmenu.hasChildNodes()) {
        var lastChild = popupmenu.lastChild;
        if (lastChild.hasAttribute("keep")) {
            break;
        }
        popupmenu.removeChild(lastChild);
    }
    var menuitem;
    var label;

    //XXX Move the handler into the command-observer...
    if (false && !projectURL && ko.sleuth.tabManager.isReady()) {
        menuitem = document.createElement("menuitem");
        label = "Run Current Test Plan"; //XXX I18N this
        menuitem.setAttribute("label", label);
        menuitem.setAttribute("class", "menuitem-iconic-wide");
        menuitem.setAttribute("oncommand",
                              "ko.sleuth.tabManager.rerunTests()");
        popupmenu.appendChild(menuitem);
    }

    menuitem = document.createElement("menuitem");
    label = "Create New Test Plan..."; //XXX I18N this
    menuitem.setAttribute("label", label);
    menuitem.setAttribute("class", "menuitem-iconic-wide");
    menuitem.setAttribute("accesskey", "C");
    menuitem.setAttribute("oncommand",
                          ("ko.sleuth.manager.createNewTestPlan('"
                           + projectURL + "')"));
    popupmenu.appendChild(menuitem);

    if (testPlans.length > 0) {
        popupmenu.appendChild(document.createElement("menuseparator"));
        for (var i = 0; i < testPlans.length; i++) {
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", testPlans[i]);
            menuitem.setAttribute("class", "menuitem-iconic-wide");
            menuitem.setAttribute("accesskey",
                                  testPlans[i].substr(0, 1).toUpperCase());
            var handler = ("ko.sleuth.manager." + methodName + "('"
                           + projectURL
                           + "', "
                           + i
                           + ");");
            menuitem.setAttribute("oncommand", handler);
            popupmenu.appendChild(menuitem);
        }
    }
};

Sleuth.prototype.addProjectTestPlanNamesToPopup = function(event) {
    return this.updateProjectTestPlanMenu(event.target);
};

Sleuth.prototype.doSelectedProjectTestPlan = function(prjURL, testPlanIndex) {
    var projectManager = ko.projects.manager;
    var project = projectManager.getProjectByURL(prjURL);
    if (projectManager.getCurrentProject() != project) {
        projectManager.setCurrentProject(project);
    }
    var testPlans = this._getProjectTestPlanNames(project, true);
    var sleuthPrefs = project.prefset.getPref(this.TESTPLAN_PREFNAME);
    ko.sleuth.tabManager.setProject(project, testPlans[testPlanIndex]);
    this._doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex);
};

Sleuth.prototype.doSelectedGlobalTestPlan = function(urlNotUsed, testPlanIndex) {
    var testPlans = this._getGlobalTestPlanNames(true);
    var sleuthPrefs = this._getGlobalPrefs().getPref(this.TESTPLAN_PREFNAME);
    ko.sleuth.tabManager.clearProject();
    ko.sleuth.tabManager.setProject(null, testPlans[testPlanIndex]);
    this._doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex);
};

 Sleuth.prototype._doSelectedTestPlan = function _doSelectedTestPlan(sleuthPrefs, testPlans, testPlanIndex) {
    var testPlanName = testPlans[testPlanIndex];
    if (!testPlanName) {
        dump("Can't find a test plan for test " + testPlanName + "\n");
        return;
    }
    var testPlanPref = sleuthPrefs.getPref(testPlanName);
    ko.sleuth.tabManager.show();
    var absPath = this.dirFromPrefs(testPlanPref);
    ko.sleuth.tabManager.selectTest(testPlanName,
                                    absPath,
                                    testPlanPref.getStringPref("language"),
                                    testPlanPref.getStringPref("command_line"));
};

Sleuth.prototype._getFirstRecognizedLanguage_CheckPart = function _getFirstRecognizedLanguage_CheckPart(part, candidates, currDepth) {
    var childrenObj = {};
    var countObj = {};
    part.getChildren(childrenObj, countObj);
    var children = childrenObj.value;
    for (var i = 0; i < children.length; i++) {
        var child = children[i];
        switch(child.type) {
        case "livefolder":
            candidates.push([currDepth, child]);
            break;
        case "file":
            var language = this.langRegistrySvc.suggestLanguageForFile(child.getFile().baseName);
            if (language in this.supported_languages) {
                return language;
            }
        }
    }
    return null;
};

Sleuth.prototype.getFirstRecognizedProjectLanguage = function sl_getFirstRecognizedProjectLanguage(project, supported_languages, maxDepth) {
    // Do a breadth-first-search walk of the project,
    // looking for filetypes we recognize
    // Set maxDepth to 0 (or a negative value) to never stop prematurely.
    // e.g.: Setting maxDepth to 1 looks only at the files at the first level
    // of the project.
    
    if (typeof(maxDepth) == "undefined") {
        maxDepth = 3;
    } else if (maxDepth == 0) {
        maxDepth = -1;
    }
    this.supported_languages = supported_languages;
    this.langRegistrySvc = (Components.classes['@activestate.com/koLanguageRegistryService;1'].
                           getService(Components.interfaces.koILanguageRegistryService));
    var candidates = [[maxDepth, project]];
    // Do a breadth-first-search on the project.
    var data;
    while (candidates.length > 0) {
        data = candidates.shift();
        if (data[0] == 0) {
            break;
        }
        var lang = this._getFirstRecognizedLanguage_CheckPart(data[1], candidates, data[0] - 1);
        if (lang) {
            return lang;
        }
    }
    return null;
    
};

// Pref handlers:
// project-pref directories will be stored relative to the project's path
// in the project's pref, if we write them in as URIs.

Sleuth.prototype.dirFromPrefs = function dirFromPrefs(prefset) {
    var directoryURI = prefset.getStringPref("directory");
    // convert from URI to filepath
    if (!directoryURI) {
        dump("Sleuth.prototype.dirFromPrefs: No directory pref in current set\n");
        return ko.window.getCwd();
    }
    var directoryPath = ko.uriparse.URIToLocalPath(directoryURI);
    //dump("read dir pref URI: "
    //     + directoryURI
    //     + ",\n local: "
    //     + directoryPath
    //     + "\n");
    return directoryPath;
};

Sleuth.prototype.setPrefToDir = function setPrefToRelativeDir(prefset, directoryPath) {
    var directoryURI = ko.uriparse.pathToURI(directoryPath);
    if (directoryURI && directoryURI[directoryURI.length - 1] != '/') {
        directoryURI += '/';
    }
    //dump("writing dir pref: "
    //     + directoryPath
    //     + ",\n uri: "
    //     + directoryURI
    //     + "\n");
    prefset.setStringPref("directory", directoryURI);
};

Sleuth.prototype.handleRunCurrentTestPlan = function handleRunCurrentTestPlan()
{
    if (ko.sleuth.tabManager.isReady()) {
        ko.sleuth.tabManager.rerunTests();
    } else {
        this.createNewGlobalTestPlan();
    }
};

})();
